/*
 * ColumnarLayout.java
 *
 * Created on 2  2001 ., 16:10
 */

package com.fn.repway;


import java.util.*;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 * @version 
 */
public class ColumnarLayout implements Layout {

    private double position;
    private double startPosition;
    private Border border;
    private double pageHeight;
    private double footerHeight;
    private Columns columns;
    private int column;
    private double startX;
    private Column[] cols;
    
    /** Creates new ColumnarLayout */
    public ColumnarLayout(Element tag) throws RepException {
        columns = new Columns(tag);
    }

    public void startPage(GenContext ctx) throws RepException {
        Page p = ctx.getGenReport().addPage(ctx.getPageFormat());
        position = border.getTop();
        Section pageHeader = ctx.getPageHeader();
        if (pageHeader != null) {
            Band band = pageHeader.generateBody(ctx);
            p.add(border.getLeft(), border.getTop(), band);
            position += pageHeader.getHeight();
        }
        startPosition = position;
        startX = border.getLeft();
        column = 0;
        startColumn(ctx);
    }
    
    public void begin(GenContext ctx) throws RepException {
        border = ctx.getBorder();
        position = ctx.getPosition();
        PageFormat pageFormat = ctx.getPageFormat();
        if (pageFormat.getOrient() == PageFormat.LANDSCAPE)
            pageHeight = pageFormat.getWidth();
        else
            pageHeight = pageFormat.getHeight();
        Section pageFooter = ctx.getPageFooter();
        if (pageFooter != null)
            footerHeight = pageFooter.getHeight();
        else
            footerHeight = 0.0;
        column = 0;
        startX = border.getLeft();
        startPosition = position;
        cols = columns.getColumns(ctx.getPageFormat().getWidth() -
            border.getLeft() - border.getRight());
    }

    private void finishColumn(GenContext ctx) throws RepException {
        Page page = ctx.getCurrentPage();
        for (Iterator i = ctx.getColumnBreaks().listIterator(); i.hasNext(); ) {
            Band b = ((Section)i.next()).generateBody(ctx);
            page.add(startX, position, b);
            position += b.getHeight();
        }
        column++;
        position = startPosition;
        if (column < columns.getCount())
            startX = cols[column].getStart() + border.getLeft();
    }

    private void startColumn(GenContext ctx) throws RepException {
        Page page = ctx.getCurrentPage();
        for (Iterator i = ctx.getColumnStarts().listIterator(); i.hasNext(); ) {
            Band band = ((Section)i.next()).generateBody(ctx);
            page.add(startX, position, band);
            position += band.getHeight();
        }
    }
    
    public void addBand(GenContext ctx, Band band) throws RepException {
        Page page = ctx.getCurrentPage();
        double h = band.getHeight();
        if (position + h >= pageHeight - footerHeight - ctx.getColumnBreaksHeight() -
                border.getBottom()) {
            if ((column < columns.getCount() - 1) && (startPosition + h <= 
                    pageHeight - footerHeight - ctx.getColumnBreaksHeight() -
                    border.getBottom())) {
                finishColumn(ctx);
                startColumn(ctx);
            } else {
                finishPage(ctx);
                startPage(ctx);
                page = ctx.getCurrentPage();
            }
        }
        page.add(startX, position, band);
        position += h;
    }
    
    public void end(GenContext ctx) throws RepException {
    }
    
    public void finishPage(GenContext ctx) throws RepException {
        Page page = ctx.getCurrentPage();
        finishColumn(ctx);
        Section pageFooter = ctx.getPageFooter();
        if (pageFooter != null) {
            Band band = pageFooter.generateBody(ctx);
            page.add(border.getLeft(), pageHeight - border.getTop() - 
                pageFooter.getHeight(), band);
        }
        position = pageHeight;
    }
}
