/*
 * Columns.java
 *
 * Created on 17  2001 ., 16:28
 */

package com.fn.repway;

import java.util.*;
import org.w3c.dom.Element;

/**
 *
 * @author  asso
 * @version 
 */
public class Columns {
    private double[] widths;
    private double[] gaps;
    private double fixedWidth;
    private Column[] columnsCache;
    private double cachedWidth;

    /** Creates new Columns */
    public Columns() {
        widths = new double[1];
        widths[0] = -1.0;
        fixedWidth = 0.0;
        columnsCache = null;
    }
    
    public Columns(Element tag) throws RepException {
        XMLIterator xmlIterator = new XMLIterator(tag);
        if (! xmlIterator.hasNext()) {
            int cnt;
            try {
                cnt = XMLUtils.getAttrib(tag, "count", 2);
            } catch (Exception e) {
                throw new RepException("Invalid columns count");
            }            
            if (cnt < 1)
                throw new RepException("Columns quantity should be 1 or greater");
            double gap = parseSize(XMLUtils.getAttrib(tag, "gap", "10.0"), "Invalid gap");
            initSimple(cnt, gap);
        } else
            initAdvanced(tag);
        columnsCache = null;
    }

    private void initSimple(int count, double gap) throws RepException {
        if (count == 1) {
            widths = new double[1];
            widths[0] = -1.0;
            fixedWidth = 0.0;
            return;
        }
            
        double cw;
        if (gap > 0) {
            fixedWidth = gap * (count - 1);
            cw = 1.0 / count;
        } else {
            fixedWidth = 0.0;
            double space = gap * (count - 1);
            if (space >= 1.0)
                throw new RepException("Too wide gap");
            cw = (1.0 - space) / count;
        }                        
        widths = new double[count];
        gaps = new double[count - 1];

        for (int i = 0; i < count; i++) {
            widths[i] = -cw;
            if (i < count - 1) gaps[i] = gap;
        }
    }

    private void initAdvanced(Element tag) throws RepException {
        int cnt = 0;
        List gList = new LinkedList();
        List wList = new LinkedList();
        boolean lastGap = true;
        boolean first = true;
        double pSum = 0.0;
        fixedWidth = 0.0;
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            String dflt = "invalid width";
            List l;
            if (subtag.getTagName().equals("gap")) {
                if (lastGap)
                    throw new RepException("Invalid gap placement");
                lastGap = true;
                dflt = "0";
                l = gList;
            } else if (subtag.getTagName().equals("column")) {
                if ((! lastGap) && (! first))
                    gList.add(new Double(0.0));
                lastGap = false;
                l = wList;
            } else 
                throw new RepException("Invalid tag in columns description"); 
            double v = parseSize(XMLUtils.getAttrib(tag, "width", dflt));
            if (v > 0) 
                fixedWidth += v;
            else
                pSum -= v;
            l.add(new Double(v));
            if (first) first = false;
        }

        cnt = wList.size();
        if (gList.size() != cnt + 1)
            throw new RepException("Internal columns calculations error");
        
        if (cnt < 1)
            cnt = 1;    /* impossible but safe */
        
        if (cnt == 1) {
            widths = new double[1];
            widths[0] = -1.0;
            fixedWidth = 0.0;
            return;
        }

        double pCor = 1.0;
        if ((pSum > 0.0) && (pSum != 1.0))
            pCor = 1.0 / pSum;
        
        widths = new double[cnt];
        gaps = new double[cnt - 1];
        
        int idx = 0;
        for (Iterator i = wList.listIterator(); i.hasNext(); idx++) {
            Double V = (Double)i.next();
            double v = V.doubleValue();
            if ((v < 0) && (pCor != 1.0))
                v = v * pSum;
            widths[idx] = v;
        }
        
        idx = 0;
        for (Iterator i = gList.listIterator(); i.hasNext(); idx++) {
            Double V = (Double)i.next();
            double v = V.doubleValue();
            if ((v < 0) && (pCor != 1.0))
                v = v * pSum;
            gaps[idx] = v;
        }
    }
    
    private double parseSize(String str) throws RepException {
        return parseSize(str, "Invalid size");
    }
    
    private double parseSize(String str, String excption) throws RepException {        
        return 0.0;
    }

    private double calcValue(double num, double width) {
        if (num >= 0)
            return num;
        else
            return -(width - fixedWidth) * num;
    }
    
    public Column[] getColumns(double totalWidth) throws RepException {
        if ((columnsCache != null) && (cachedWidth == totalWidth))
            return columnsCache;
        
        int count = getCount();
        Column columns[];
        if (count == 1) {
            columns = new Column[1];
            columns[0] = new Column(0, totalWidth);
        } else {
            double start = 0.0, cw = 0.0;;
            columns = new Column[count];
            for (int i = 0; i < count; i++) {
                if (i > 0)
                    start += calcValue(gaps[i - 1], totalWidth) + cw;
                cw = calcValue(widths[i], totalWidth);
                columns[i] = new Column(start, cw);
            }
        }
        
        columnsCache = columns;
        cachedWidth = totalWidth;
        return columns;
    }
    
    public int getCount() {
        return widths.length;
    }
}
