/*
 * CompLabel.java
 *
 * Created on 5  2001 ., 12:00
 */

package com.fn.repway;


import java.io.IOException;
import java.io.Writer;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 * @version 
 */
public class CompLabel extends Primitive {

    protected String text;
    protected java.awt.Font font;
    protected java.awt.Color color;
    protected int align;
    protected boolean clip;
    protected double rotation;

    /** Creates new CompLabel */
    public CompLabel(double x, double y, double width, double height, java.awt.Font font,
            java.awt.Color color, int align, boolean clip, String text, double rotate) {
        super(x, y, width, height);
        this.text = text;
        this.font = font;
        this.color = color;
        this.align = align;
        this.clip = clip;
        if ((width <= 0.0) || (height <= 0.0))
            this.clip = false;
        rotation = rotate;
    }
    
    public CompLabel(Element tag) throws RepException {
        super(tag);
        color = XMLUtils.getColorAttrib(tag, "color", "#000000");
        text = XMLUtils.getData(tag);
        align = XMLUtils.getAttrib(tag, "align", 0);
        rotation = XMLUtils.getAttrib(tag, "rotation", 0.0);
        clip = XMLUtils.getAttrib(tag, "clip", false);
        String fontName = XMLUtils.getAttrib(tag, "fontName", "Times");
        int fontStyle = XMLUtils.getAttrib(tag, "fontStyle", 0);
        int fontSize = XMLUtils.getAttrib(tag, "fontSize", 0);
        font = new java.awt.Font(fontName, fontStyle, fontSize);
    }

    void draw(java.awt.Graphics2D g2) {
        g2.setFont(font);
        float x = (float)left;
        java.awt.FontMetrics fm = g2.getFontMetrics(font);
        float y = (float)top + fm.getAscent();
        float alignedX = x;
        float alignedY = y;
        java.awt.geom.Rectangle2D boundsRect = fm.getStringBounds(text, g2);
        double textWidth = boundsRect.getWidth();
        double textHeight = boundsRect.getHeight();
        if (align != Label.ALIGN_LEFT) {
            if (align == Label.ALIGN_RIGHT)
                alignedX = (float)(x + width - textWidth);
            else
                alignedX = (float)(x + (width - textWidth) / 2.0);
        }
        java.awt.Shape oldClip = null;
        if (clip) {
            oldClip = g2.getClip();
            java.awt.geom.Rectangle2D r = new java.awt.geom.Rectangle2D.Double(left, top, width, height);
            g2.clip(r);
        }
        g2.setPaint(color);
        java.awt.geom.AffineTransform oldTransform = g2.getTransform();
        if (rotation != 0.0) {
            g2.translate(x, y);
            alignedX = 0; //(float)(textWidth / 2.0); 
            alignedY = 0; //(float)(textHeight / 2.0);
            g2.rotate(rotation / 180.0 * Math.PI);
        }
        g2.drawString(text, alignedX, alignedY);
        if (clip)
            g2.setClip(oldClip);
        if (rotation != 0.0)
            g2.setTransform(oldTransform);
    }    
    
    public void save(Writer out) throws IOException {
        out.write("<label");
        saveRectAttribs(out);
        out.write("color=\"");
        out.write(Utils.colorToStr(color));
        out.write("\" fontName=\"");
        out.write(XMLUtils.escape(font.getFontName()));
        out.write("\" fontSize=\"");
        out.write(Integer.toString(font.getSize()));
        out.write("\" fontStyle=\"");
        out.write(Integer.toString(font.getStyle()));
        out.write("\" align=\"");
        out.write(Integer.toString(align));
        out.write("\" rotation=\"");
        out.write(Double.toString(rotation));
        out.write("\" clip=\"");
        out.write(clip ? "on" : "off");
        out.write("\">");
        out.write(XMLUtils.escape(text));
        out.write("</label>");
    }
    
}
