/*
 * CompPoly.java
 *
 * Created on 10  2003 ., 8:38
 */

package com.fn.repway;


import java.awt.Color;
import java.awt.BasicStroke;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class CompPoly extends Primitive {

    private Color frameColor;
    private Color fillColor;
    private double thickness;
    private GeneralPath polygon;
    private double deltaX = 0.0;
    private double deltaY = 0.0;
    private double xCoords[];
    private double yCoords[];
    
    /** Creates a new instance of CompPoly */
    public CompPoly(double xPoints[], double yPoints[], double thickness, 
            Color frameColor, Color fillColor) 
    {
        this.frameColor = frameColor;
        this.fillColor = fillColor;
        this.thickness = thickness;
        
        xCoords = new double[xPoints.length];
        System.arraycopy(xPoints, 0, xCoords, 0, xPoints.length);
        yCoords = new double[yPoints.length];
        System.arraycopy(yPoints, 0, yCoords, 0, yPoints.length);
        
        buildGeneralPath();
    }

    public CompPoly(Element tag) throws RepException {
        super(tag);
        frameColor = XMLUtils.getColorAttrib(tag, "frameColorr", "#000000");
        fillColor = XMLUtils.getColorAttrib(tag, "fillColor", "#FFFFFF");
        thickness = XMLUtils.getAttrib(tag, "thickness", 1.0);
        List lx = new LinkedList();
        List ly = new LinkedList();
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            lx.add(new Double(XMLUtils.getAttrib(subtag, "x", 0.0)));
            ly.add(new Double(XMLUtils.getAttrib(subtag, "y", 0.0)));
        }
        xCoords = listToDoubleArray(lx);
        yCoords = listToDoubleArray(ly);
        buildGeneralPath();
    }
    
    private void buildGeneralPath() {
        double minX = 0.0, minY = 0.0, maxX = 0.0, maxY = 0.0;
        int len = xCoords.length;
        polygon = new GeneralPath(GeneralPath.WIND_EVEN_ODD, len);
        polygon.moveTo((float)xCoords[0], (float)yCoords[0]);
        for (int i = 0; i < len; i++) {
            double x = xCoords[i];
            double y = yCoords[i];
            if (i == 0) {
                minX = maxX = x;
                minY = maxY = y;
            } else {
                if (x < minX) minX = x;
                if (x > maxX) maxX = x;
                if (y < minY) minY = y;
                if (y > maxY) maxY = y;
                polygon.lineTo((float)x, (float)y);
            }
        }
        polygon.closePath();
        
        left = minX;
        top = minY;
        width = maxX - minX;
        height = maxY - minY;
    }
    
    private double[] listToDoubleArray(List list) {
        double arr[] = new double[list.size()];
        int k = 0;
        for (Iterator i = list.iterator(); i.hasNext(); k++)
            arr[k] = ((Double)i.next()).doubleValue();
        return arr;
    }
    
    public void moveBy(double deltaX, double deltaY) {
        super.moveBy(deltaX, deltaY);
        this.deltaX += deltaX;
        this.deltaY += deltaY;
    }
    
    void draw(java.awt.Graphics2D g2) {
        AffineTransform oldTransform = g2.getTransform();
        g2.translate(deltaX, deltaY);
        Stroke s = new BasicStroke((float)thickness);
        Stroke oldStroke = g2.getStroke();
        g2.setStroke(s);
        g2.setColor(fillColor);
        g2.fill(polygon);
        g2.setColor(frameColor);
        g2.draw(polygon);
        g2.setStroke(oldStroke);
        g2.setTransform(oldTransform);
    }
    
    public void save(java.io.Writer out) throws java.io.IOException {
        out.write("<poly");
        saveRectAttribs(out);
        out.write("frameColor=\"");
        out.write(Utils.colorToStr(frameColor));
        out.write("\" fillColor=\"");
        out.write(Utils.colorToStr(fillColor));
        out.write("\" thickness=\"");
        out.write(Double.toString(thickness));
        out.write("\">");
        for (int i = 0; i < xCoords.length; i++) {
            out.write("<vert x=\"");
            out.write(Double.toString(xCoords[i] + deltaX));
            out.write("\" y=\"");
            out.write(Double.toString(yCoords[i] + deltaY));
            out.write("\"/>");
        }
        out.write("</poly>");
    }
    
}
