/*
 * FlatHistDrawer.java
 *
 * Created on 26  2003 ., 14:39
 */

package com.fn.repway;


import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class FlatHistDrawer implements HistDrawer {
    
    private double axisThickness;
    private Color axisColor;
    private double axisTickThickness;
    private Color axisTickColor;
    private double axisTickSize;
    private Label tickLabel;
    private double tickLabelGap;
    
    /** Creates a new instance of FlatHistDrawer */
    public FlatHistDrawer(Element tag) throws RepException {
        Element subtag = XMLUtils.getSubTag(tag, "style");
        axisThickness = XMLUtils.getAttrib(subtag, "axisThickness", 0.5);
        axisColor = XMLUtils.getColorAttrib(subtag, "axisColor", "#000000");
        axisTickThickness = XMLUtils.getAttrib(subtag, "tickThickness", 0.4);
        axisTickSize = XMLUtils.getAttrib(subtag, "tickSize", 2.0);
        axisTickColor = XMLUtils.getColorAttrib(subtag, "tickColor", "#000000");
        tickLabelGap = XMLUtils.getAttrib(subtag, "tickLabelGap", 1.0);
        try {
            Element labelTag = XMLUtils.getSubTag(subtag, "tickLabel");
            if (labelTag != null)
                tickLabel = new Label(labelTag);
        } catch (Exception e) { }
    }
    
    public void drawAxis(GenContext ctx, Band band, boolean front, Axis axis, Rectangle2D.Double area, double koof) 
    {
        band.add(front, new CompLine(area.x, area.y + area.height, area.width, 0.0, 
            axisThickness, axisColor));
        band.add(front, new CompLine(area.x, area.y, 0.0, area.height, 
            axisThickness, axisColor));
        
        double tickStep = axis.getTickInterval();
        if (tickStep > 0.0) {
            double max = axis.getMaxValue();
            for (double i = 0.0; i <= max; i += tickStep) {
                band.add(front, new CompLine(area.x, area.y + area.height - i*koof, 
                    -axisTickSize, 0.0, axisTickThickness, axisTickColor));
                if (tickLabel != null) {
                    ctx.setVariable("__axisTick", new Value(i));
                    tickLabel.setLeft(area.x - axisTickSize - tickLabel.getWidth() - tickLabelGap);
                    tickLabel.setTop(area.y + area.height - i*koof - tickLabel.getHeight() / 2.0);
                    try { tickLabel.generate(ctx, band, front); } catch (Exception e) { }
                }
            }
        }
    }
    
    public void drawBar(GenContext ctx, Band band, boolean front, Axis axisY, Rectangle2D.Double area, 
            double position, double width, double value, double thickness, Color border, Color fill) 
    {
        band.add(front, new CompBar(area.x+position, area.y + area.height - value, 
                width, value, thickness, border, fill));
    }
        
}
