/*
 * GenContext.java
 *
 * Created on 14  2001 ., 10:03
 */

package com.fn.repway;

import java.util.*;

/**
 *
 * @author  asso
 * @version 
 */
public class GenContext {
    
    private GeneratedReport generatedReport;
    private double position;
    private DataSource dataSource;
    private int side;
    private LinkedList layouts = new LinkedList();
    private LinkedList pageHeaders = new LinkedList();
    private LinkedList pageFooters = new LinkedList();
    private LinkedList pageFormats = new LinkedList();
    private LinkedList borders = new LinkedList();
    private LinkedList pageBreaks = new LinkedList();
    private LinkedList pageStarts = new LinkedList();
    private LinkedList columnBreaks = new LinkedList();
    private LinkedList columnStarts = new LinkedList();
    private double pageBreaksHeight = 0.0;
    private double columnBreaksHeight = 0.0;
    private Map parameters = new HashMap();
    private Report report;

    /** Creates new GenContext */
    public GenContext(GeneratedReport gr, DataSource dataSource, int side, Map params, Report rep) {
        generatedReport = gr;
        report = rep;
        position = 0.0;
        this.dataSource = dataSource;
        this.side = side;
        if (params != null)
            parameters.putAll(params);
    }

    public GeneratedReport getGenReport() {
        return generatedReport;
    }
    
    public Page getCurrentPage() {
        return generatedReport.getLastPage();
    }
    
    public double getPosition() {
        return position;
    }
    
    public void setPosition(double pos) {
        position = pos;
    }
    
    public DataSource getDataSource() {
        return dataSource;
    }
    
    public DataSet getDataSet(String path) throws RepException {
        DataSet ds = null;
        StringTokenizer st = new StringTokenizer(path, "::", false);
        while (st.hasMoreTokens()) {
            String s = st.nextToken();
            if (! st.hasMoreTokens()) {
                if (ds == null)
                    return dataSource.getDataSet(s);
                else
                    return ds.getValue(s).getAsDataSet();
            } else {
                if (ds == null)
                    ds = dataSource.getDataSet(s);
                else
                    ds = ds.getValue(s).getAsDataSet();
                if (ds == null)
                    return null;
            }
        }
        return null;
    }
    
    public int getSide() {
        return side;
    }

    public Layout getLayout() {
        if (layouts.size() > 0)
            return (Layout)layouts.getLast();
        else
            return null;
    }
    
    public void pushLayout(Layout layout) {
        if (layout != null)
            layouts.addLast(layout);
    }
    
    public void popLayout(Layout layout) {
        if ((layout != null) && (layouts.size() > 0))
            layouts.removeLast();
    }

    public Section getPageHeader() {
        if (pageHeaders.size() > 0)
            return (Section)pageHeaders.getLast();
        else
            return null;
    }
    
    public void pushPageHeader(Section section) {
        if (section != null)
            pageHeaders.addLast(section);
    }
    
    public void popPageHeader(Section section) {
        if ((section != null) && (pageHeaders.size() > 0))
            pageHeaders.removeLast();
    }

    public Section getPageFooter() {
        if (pageFooters.size() > 0)
            return (Section)pageFooters.getLast();
        else
            return null;
    }
    
    public void pushPageFooter(Section section) {
        if (section != null)
            pageFooters.addLast(section);
    }
    
    public void popPageFooter(Section section) {
        if ((section != null) && (pageFooters.size() > 0))
            pageFooters.removeLast();
    }
    
    public Border getBorder() {
        if (borders.size() > 0)
            return (Border)borders.getLast();
        else
            return null;
    }
    
    public void pushBorder(Border border) {
        if (border != null)
            borders.addLast(border);
    }
    
    public void popBorder(Border border) {
        if ((border != null) && (borders.size() > 0))
            borders.removeLast();
    }
    
    public PageFormat getPageFormat() {
        if (pageFormats.size() > 0)
            return (PageFormat)pageFormats.getLast();
        else
            return null;
    }
    
    public void pushPageFormat(PageFormat pageFormat) {
        if (pageFormat != null)
            pageFormats.addLast(pageFormat);
    }
    
    public void popPageFormat(PageFormat pageFormat) {
        if ((pageFormat != null) && (pageFormats.size() > 0))
            pageFormats.removeLast();
    }

    public List getPageBreaks() {
        return pageBreaks;
    }
    
    public void pushPageBreak(Section section) {
        if (section != null) {
            pageBreaksHeight += section.getHeight();
            pageBreaks.addLast(section);
        }
    }
    
    public void popPageBreak(Section section) {
        if ((section != null) && (pageBreaks.size() > 0)) {
            pageBreaksHeight -= section.getHeight();
            pageBreaks.removeLast();
        }
    }
    
    public double getPageBreaksHeight() {
        return pageBreaksHeight;
    }

    public List getPageStarts() {
        return pageStarts;
    }
    
    public void pushPageStart(Section section) {
        if (section != null)
            pageStarts.addLast(section);
    }
    
    public void popPageStart(Section section) {
        if ((section != null) && (pageStarts.size() > 0))
            pageStarts.removeLast();
    }

    public List getColumnBreaks() {
        return columnBreaks;
    }
    
    public void pushColumnBreak(Section section) {
        if (section != null) {
            columnBreaksHeight += section.getHeight();
            columnBreaks.addLast(section);
        }
    }
    
    public void popColumnBreak(Section section) {
        if ((section != null) && (columnBreaks.size() > 0)) {
            columnBreaksHeight -= section.getHeight();
            columnBreaks.removeLast();
        }
    }

    public double getColumnBreaksHeight() {
        return columnBreaksHeight;
    }

    public List getColumnStarts() {
        return columnStarts;
    }
    
    public void pushColumnStart(Section section) {
        if (section != null)
            columnStarts.addLast(section);
    }
    
    public void popColumnStart(Section section) {
        if ((section != null) && (columnStarts.size() > 0))
            columnStarts.removeLast();
    }
    
    public void setVariable(String name, Value value) {
        parameters.put(name, value);
    }
    
    public Value getValue(String name) throws RepException {
        if (name.equals("pagenum"))
            return new Value(generatedReport.getNumberOfPages());
        else {
            Value v = (Value)parameters.get(name);
            if (v != null)
                return v;
            else {
                DataSet ds = getDataSource().getDataSet(name);
                if (ds != null)
                    return new Value(ds);
                String field = null;
                StringTokenizer st = new StringTokenizer(name, "::", false);
                while (st.hasMoreTokens()) {
                    String s = st.nextToken();
                    if (st.hasMoreTokens()) {
                        if (ds == null)
                            ds = getDataSource().getDataSet(s);
                        else
                            ds = ds.getValue(s).getAsDataSet();
                        if (ds == null)
                            break;
                    } else {
                        field = s;
                    }
                }
                if ((ds == null) || (field == null))
                    throw new RepException("Can't find value '" + name + "'");
                return ds.getValue(field);
            }
        }
    }
    
    public Report getReport() {
        return report;
    }
}
