/*
 * Hystogram.java
 *
 * Created on 6  2003 ., 20:31
 */

package com.fn.repway;


import java.awt.geom.Rectangle2D;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class Hystogram extends RepElement {
    
    private String dataSetName;
    private List fields;
    private double barsGap;
    private double groupsGap;
    private Rectangle2D.Double graphArea;
    private Axis axis;
    private HistDrawer drawer;
    private boolean resetData;
    
    /** Creates a new instance of Hystogram */
    public Hystogram(Element tag) throws RepException {
        super(tag);
        fields = new LinkedList();
        dataSetName = XMLUtils.getAttrib(tag, "dataset", (String)null);
        if (dataSetName == null)
            throw new RepException("Histogram object requires dataset attribute");
        Element subtag = XMLUtils.getSubTag(tag, "fields");
        XMLIterator i = new XMLIterator(subtag);
        while (i.hasNext())
            fields.add(new Field(i.next()));
        barsGap = XMLUtils.getAttrib(tag, "barGap", 1.0);
        groupsGap = XMLUtils.getAttrib(tag, "groupsGap", 2.0);
        graphArea = XMLUtils.getRectAttribs(XMLUtils.getSubTag(tag, "graphArea"),
                0.0, 0.0, width, height, left, top);
        axis = new Axis(XMLUtils.getSubTag(tag, "axis"));
        String style = XMLUtils.getAttrib(tag, "style", "flat");
        drawer = HistDrawerFactory.getAxisDrawer(style, tag);
        resetData = XMLUtils.getAttrib(tag, "resetData", false);
    }

    private class Field {
        private String name;
        private String description;
        private java.awt.Color frameColor;
        private java.awt.Color fillColor;
        private Label topLabel;
        private Label bottomLabel;
        private String group;
        private double thickness;
        private DataField topField;
        private DataField bottomField;
        
        public Field(Element tag) throws RepException {
            if (! tag.getNodeName().equals("field"))
                throw new RepException("<field> expected but '" + tag.getNodeName() + "' found");
            name = XMLUtils.getAttrib(tag, "name", "");
            description = XMLUtils.getAttrib(tag, "description", "");
            frameColor = XMLUtils.getColorAttrib(tag, "frame", "#000000");
            fillColor = XMLUtils.getColorAttrib(tag, "color", "#ff0000");
            Element subtag = XMLUtils.getSubTag(tag, "top");
            if (subtag != null)
                topLabel = new Label(subtag);
            subtag = XMLUtils.getSubTag(tag, "bottomLabel");
            if (subtag != null)
                bottomLabel = new Label(subtag);
            subtag = XMLUtils.getSubTag(tag, "topLabel");
            if (subtag != null)
                topLabel = new Label(subtag);
            group = XMLUtils.getAttrib(tag, "group", "");
            thickness = XMLUtils.getAttrib(tag, "thickness", 0.5);
            
            subtag = XMLUtils.getSubTag(tag, "topValue");
            if (subtag != null) {
                String type = XMLUtils.getAttrib(subtag, "type", "");
                topField = FieldFactory.createField(type, subtag);
            }
            subtag = XMLUtils.getSubTag(tag, "bottomValue");
            if (subtag != null) {
                String type = XMLUtils.getAttrib(subtag, "type", "");
                bottomField = FieldFactory.createField(type, subtag);
            }
        }
        
        public String getName() {
            return name;
        }
        
        public String getDescription() {
            return description;
        }
        
        public java.awt.Color getFillColor() {
            return fillColor;
        }
        
        public java.awt.Color getFrameColor() {
            return frameColor;
        }
        
        public Label getTopLabel() {
            return topLabel;
        }
        
        public Label getBottomLabel() {
            return bottomLabel;
        }
        
        public String getGroup() {
            return group;
        }
        
        public double getThickness() {
            return thickness;
        }

        public DataField getTopField() {
            return topField;
        }

        public DataField getBottomField() {
            return bottomField;
        }
    }

    private class DataEntry {
        private Value value;
        private Field field;
        private Value topValue;
        private Value bottomValue;

        public DataEntry(Field f, Value v, Value top, Value bottom) {
            value = v;
            field = f;
            topValue = top;
            bottomValue = bottom;
        }

        public Value getValue() {
            return value;
        }

        public Field getField() {
            return field;
        }

        public Value getTopValue() {
            return topValue;
        }

        public Value getBottomValue() {
            return bottomValue;
        }
    }

    private List getExistedFields(GenContext ctx) throws RepException {
        List list = new LinkedList();
        for (Iterator i = fields.iterator(); i.hasNext(); ) {
            Field f = (Field)i.next();
            try {
                Value v = ctx.getValue(f.getName());
                if (v != null)
                    list.add(f);
            } catch (Exception e) { }
        }
        return list;
    }

    private List getData(GenContext ctx, DataSet dataSet, List usedFields, 
            double[] maxValues) throws RepException 
    {
        int fieldsCount = usedFields.size();
        List data = new LinkedList();
        
        double maxValue = 0;
        boolean first = true;
        
        do {
            DataEntry entries[] = new DataEntry[fieldsCount];
            int no = 0;
            for (Iterator i = usedFields.iterator(); i.hasNext(); no++) {
                Field f = (Field)i.next();
                DataField topField = f.getTopField();
                DataField bottomField = f.getBottomField();
                entries[no] = new DataEntry(f, ctx.getValue(f.getName()),
                        topField != null ? topField.getText(ctx) : null, 
                        bottomField != null ? bottomField.getText(ctx): null );
                if (first)
                    maxValues[no] = entries[no].getValue().getAsDouble();
                else {
                    if (maxValues[no] < entries[no].getValue().getAsDouble())
                        maxValues[no] = entries[no].getValue().getAsDouble();
                }
            }
            if (first)
                first = false;
            data.add(entries);
        } while (dataSet.gotoNext());
        
        int no = 0;
        for (Iterator i = usedFields.iterator(); i.hasNext(); no++) {
            Field f = (Field)i.next();
            int nno = 0;
            for (Iterator j  = usedFields.iterator(); j.hasNext(); nno++) {
                Field nf = (Field)j.next();
                if (nno != no) {
                    if (f.getGroup().equals(nf.getGroup())) {
                        if (maxValues[no] > maxValues[nno])
                            maxValues[no] = maxValues[nno];
                        else
                            maxValues[nno] = maxValues[no];
                    }
                }
            }
        }
        
        return data;
    }

    private class ValueField implements DataField {
        private Value value;
        
        public ValueField(Value v) {
            value = v;
        }
        
        public Value getText(GenContext ctx) throws RepException {
            return value;
        }        
    }
    
    public void generate(GenContext ctx, Band band, boolean front) throws RepException {
        DataSet ds = null;
        if ((dataSetName != null) && (! dataSetName.equals("")))
            ds = ctx.getDataSet(dataSetName);
        if (ds == null)
            return;
        
        if (resetData)
            ds.reset();
        
        if ((! ds.hasCurrent()))
            return;
        
        List realFields = getExistedFields(ctx);
        int fieldsCount = realFields.size();
        if (fieldsCount < 1)
            return;
        
        double maxValues[] = new double[fieldsCount];
        List data = getData(ctx, ds, realFields, maxValues);
        double multKoofs[] = new double[fieldsCount];
        axis.updateMinMax(0.0);
        for (int i = 0; i < fieldsCount; i++) {
            multKoofs[i] = graphArea.getHeight() / maxValues[i];
            axis.updateMinMax(maxValues[i]);
        }
        drawer.drawAxis(ctx, band, front, axis, graphArea, multKoofs[0]);
        
        int recordsCount = data.size();
        double groupWidth = (graphArea.getWidth() - groupsGap*((double)recordsCount-1.0)) / (double)recordsCount;
        double barWidth = (groupWidth - barsGap*((double)fieldsCount-1.0)) / (double)fieldsCount;
        
        double pos = 0.0;
        for (Iterator i = data.iterator(); i.hasNext(); ) {
            DataEntry values[] = (DataEntry[])i.next();
            for (int j = 0; j < values.length; j++) {
                Field field = values[j].getField();
                double h = values[j].getValue().getAsDouble() * multKoofs[j];
                drawer.drawBar(ctx, band, front, axis, graphArea, 
                        pos, barWidth, h, field.getThickness(), 
                        field.getFrameColor(), field.getFillColor());
                Label topLabel = field.getTopLabel();
                Value topValue = values[j].getTopValue();
                if ((topLabel != null) && (topValue != null)) {
                    topLabel.setDataField(new ValueField(topValue));
                    topLabel.setLeft(graphArea.x + pos);
                    topLabel.setTop(graphArea.y + graphArea.height - h /*- topLabel.getHeight()*/);
                    topLabel.setWidth(barWidth);
                    topLabel.generate(ctx, band, front);
                }
                Label bottomLabel = field.getBottomLabel();
                Value bottomValue = values[j].getBottomValue();
                if ((bottomLabel != null) && (bottomValue != null)) {
                    bottomLabel.setDataField(new ValueField(bottomValue));
                    bottomLabel.setLeft(graphArea.x + pos);
                    bottomLabel.setTop(graphArea.y + graphArea.height);
                    bottomLabel.setWidth(barWidth);
                    bottomLabel.generate(ctx, band, front);
                }
                pos += barWidth;
                if (j != fieldsCount - 1)
                    pos += barsGap;
            }
            pos += groupsGap;
        }
    }
}
