/*
 * JdbcDataSet.java
 *
 * Created on 15  2003 ., 16:09
 */

package com.fn.repway;


import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;


/**
 *
 * @author  asso
 */
public class JdbcDataSet implements DataSet {
    
    private String name;
    private ResultSet resultSet;
    private boolean hasCurrent;
    private Map columns;
    
    /** Creates a new instance of JdbcDataSet */
    public JdbcDataSet(String name, ResultSet resultSet) {
        this.name = name;
        this.resultSet = resultSet;
        init();
    }
    
    private void init() {
        if (resultSet == null)
            hasCurrent = false;
        else
            try {
                hasCurrent = resultSet.next();
                ResultSetMetaData metaData = resultSet.getMetaData();
                columns = new HashMap();
                int cnt = metaData.getColumnCount();
                for (int i = 1; i <= cnt; i++) {
                    columns.put(metaData.getColumnName(i), new Column(i,
                            mapType(metaData.getColumnType(i))));
                }
            } catch (Exception e) {
                hasCurrent = false;
            }
    }
    
    private class Column {
        private final int index;
        private final int type;
        public Column(int idx, int t) {
            index = idx;
            type = t;
        }
    }
    
    private int mapType(int jdbcType) {
        switch (jdbcType) {
            case Types.BOOLEAN:
            case Types.SMALLINT:
            case Types.INTEGER:
            case Types.BIT: return Value.INTEGER;
            case Types.DECIMAL:
            case Types.NUMERIC:
            case Types.BIGINT:
            case Types.FLOAT:
            case Types.DOUBLE: return Value.DOUBLE;
            case Types.DATE: return Value.DATE;
            case Types.TIME: return Value.TIME;
            case Types.TIMESTAMP: return Value.DATETIME;
            default: return Value.STRING;
        }
    }
    
    public String getName() {
        return name;
    }
    
    public Value getValue(String name) throws RepException {
        try {
            Column col = (Column)columns.get(name);
            if (col == null)
                throw new RepException("Invalid column name");
            switch (col.type) {
                case Value.INTEGER: return new Value(resultSet.getInt(col.index));
                case Value.DOUBLE: return new Value(resultSet.getDouble(col.index));
                case Value.DATE: return new Value(col.type, resultSet.getDate(col.index));
                case Value.TIME: return new Value(col.type, resultSet.getTime(col.index));
                case Value.DATETIME: return new Value(col.type, resultSet.getTimestamp(col.index));
                default: return new Value(resultSet.getString(col.index));
            }
        } catch (Exception e) {
            throw new RepException("Error fetching value from JDBC: " + e.getMessage());
        }
    }
    
    public boolean gotoNext() throws RepException {
        try {
            hasCurrent = resultSet.next();
            return hasCurrent;
        } catch (Exception e) {
            hasCurrent = false;
            throw new RepException("Error moving to next JDBC record: " + e.getMessage());
        }
    }
    
    public boolean hasCurrent() throws RepException {
        return hasCurrent;
    }    
    
    public void reset() throws RepException {
        if (null == resultSet)
            return;
        try {
            resultSet.beforeFirst();
        } catch (Exception e) {
            throw new RepException("reset method is not supported in JDBC data set: "
                    + e.getMessage());
        }
        init();
    }
    
}
