/*
 * PageFormat.java
 *
 * Created on 17  2001 ., 13:31
 */

package com.fn.repway;

import java.io.IOException;
import java.io.Writer;
import java.util.ResourceBundle;
import org.w3c.dom.Element;

/**
 *
 * @author  asso
 * @version 
 */
public class PageFormat {

    public static final int LANDSCAPE = 0;
    public static final int PORTRAIT = 1;
    
    private double width = 210.0;
    private double height = 297.0;
    private int orientation = PORTRAIT;
    
    /** Creates new PageFormat */
    public PageFormat() {
    }
        
    public PageFormat(double width, double height, int orientation) {
        this.width = width;
        this.height = height;
        this.orientation = orientation;
    }
    
    public PageFormat(Element tag) throws RepException {
        parsePageFormat(tag);
    }
    
    private void parsePageFormat(Element tag) throws RepException {
        String s = XMLUtils.getAttrib(tag, "width", null);
        if (s == null) {        
            String name = XMLUtils.getAttrib(tag, "format", "a4");
            if (name.equals(""))
                throw new RepException("Invalid paper format");
            try {
                String resName = name.replace(' ', '_').toLowerCase();
                ResourceBundle bundle = ResourceBundle.getBundle("com.fn.repway.papers");
                width = Double.valueOf(bundle.getString(resName + ".width")).doubleValue();
                height = Double.valueOf(bundle.getString(resName + ".height")).doubleValue();
            } catch (Exception e) {
                throw new RepException("Invalid page format '" + name + "'");
            }
        } else {
            width = XMLUtils.getAttrib(tag, "width", 0.0);
            height = XMLUtils.getAttrib(tag, "height", 0.0);
        }
            
        String orient = XMLUtils.getAttrib(tag, "orientation", "portrait");
        String name = orient.toLowerCase();
        if (name.equals("landscape") || name.equals("0"))
            orientation = LANDSCAPE;
        else if (name.equals("portrait") || name.equals("1"))
            orientation = PORTRAIT;
        else
            throw new RepException("Invalid orientation of paper '" + orient + "'");
    }
    
    public double getWidth() {
        return width;
    }
    
    public double getHeight() {
        return height;
    }
    
    public int getOrient() {
        if (orientation == -1)
            return LANDSCAPE;
        else
            return orientation;
    }
    
    public void save(Writer out) throws IOException {
        out.write("<pageformat width=\"");
        out.write(Double.toString(width));
        out.write("\" height=\"");
        out.write(Double.toString(height));
        out.write("\" orientation=\"");
        out.write(Integer.toString(getOrient()));
        out.write("\"/>");
    }
}
