/*
 * SectionsView.java
 *
 * Created on 21  2003 ., 14:36
 */

package com.fn.repway.designer;


import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;

import com.fn.repway.Part;


/**
 *
 * @author  asso
 */
public class PartView extends javax.swing.JComponent implements javax.swing.Scrollable {
    
    private Part part;
    private Draggable dragging = null;
    private SectionSeparator sectionSeparator;
    
    /** Creates a new instance of SectionsView */
    public PartView(Part part) {
        this.part = part;
        sectionSeparator = new SectionSeparator(20);
        addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                onMouseDragged(e);
            }
        });
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                onMousePressed(e);
            }
            public void mouseReleased(MouseEvent e) {
                onMouseReleased(e);
            }
        });
    }

    public Dimension getPreferredScrollableViewportSize() {
        return getPreferredSize();
    }

    public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, 
            int direction) 
    {
        return 40;
    }

    public boolean getScrollableTracksViewportHeight() {
        return true;
    }

    public boolean getScrollableTracksViewportWidth() {
        return true;
    }

    public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, 
            int direction) 
    {
        return 10;
    }

    private void onMouseDragged(MouseEvent e) {
        if (dragging != null) {
            dragging.onDragging(e);
            paintImmediately(0, 0, getWidth(), getHeight());
        }
    }
    
    private void onMousePressed(MouseEvent e) {
        if (e.getButton() == MouseEvent.BUTTON1) {
            int mY = e.getY();
            if ((mY >= sectionSeparator.getPosition()) && 
                    (mY <= sectionSeparator.getPosition() + sectionSeparator.getHeight()))
            {
                dragging = sectionSeparator;
                sectionSeparator.onDragBegin(e);
            }
        }
    }

    private void onMouseReleased(MouseEvent e) {
        if (e.getButton() == MouseEvent.BUTTON1) {
            if (dragging != null) {
                dragging.onDragEnd(e);
                dragging = null;
            }
        }
    }

    public void paintComponent(java.awt.Graphics g) {
        java.awt.Graphics2D g2 = (java.awt.Graphics2D)g;
        java.awt.Dimension d = getSize();
        
        g2.setPaint(getBackground());
        g2.fill(new java.awt.Rectangle(0, 0, d.width, d.height));
        sectionSeparator.paint(this, g);
    }
}
